using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;

namespace Bespoke.Common.Net
{
    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void TcpConnectedHandler(object sender, TcpConnectedEventArgs e);

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void TcpDataSentHandler(object sender, TcpDataSentEventArgs e);

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void TcpDataReceivedHandler(object sender, TcpDataReceivedEventArgs e);

    /// <summary>
    /// 
    /// </summary>
    public class TcpServer : IDisposable
    {
        #region Events

        /// <summary>
        /// 
        /// </summary>
        public event TcpConnectedHandler Connected;

        /// <summary>
        /// 
        /// </summary>
        public event TcpConnectedHandler Disconnected;

        /// <summary>
        /// 
        /// </summary>
        public event TcpDataReceivedHandler DataReceived;

        /// <summary>
        /// 
        /// </summary>
        public event TcpDataSentHandler DataSent;

        #endregion

        /// <summary>
        /// 
        /// </summary>
        public int Port
        {
            get
            {
                return mPort;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public IPAddress IPAddress
        {
            get
            {
                return mIPAddress;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int ActiveConnections
        {
            get
            {
                return mClientConnections.Count;
            }
        }

        /// <summary>
        /// Binds the server to the loopback address.
        /// </summary>
        /// <param name="port"></param>
        public TcpServer(int port)
            : this(IPAddress.Loopback, port)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ipAddress"></param>
        /// <param name="port"></param>
        public TcpServer(IPAddress ipAddress, int port)
        {
            mPort = port;
            mIPAddress = ipAddress;
            mClientConnections = new List<TcpConnection>();
            mConnectionsToRemove = new List<TcpConnection>();
            mAcceptingConnections = true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public static IPAddress[] GetLocalIPAddress()
        {
            IPAddress[] localAddresses = Dns.GetHostAddresses(Dns.GetHostName());
            if (localAddresses.Length == 0)
            {
                throw new Exception("No local IP Address address found.");
            }

            return localAddresses;
        }

        /// <summary>
        /// 
        /// </summary>
        public void Start()
        {
            TcpListener listener = null;

			try
			{
				mIsShuttingDown = false;
				listener = new TcpListener(mIPAddress, mPort);
				Socket workerSocket;

				listener.Start(MaxPendingConnections);

				while (mAcceptingConnections)
				{
					if (listener.Pending())
					{
						workerSocket = listener.AcceptSocket();
						Thread workerThread = new Thread(RunWorker);
						workerThread.Start(workerSocket);
					}
				}
			}
            finally
            {
                if (listener != null)
                {
                    listener.Stop();
                }

                mIsShuttingDown = true;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public void Stop()
        {
            mAcceptingConnections = false;            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="stream"></param>
        public void SendToAllClients(MemoryStream stream)
        {
            SendToAllClients(stream.GetBuffer(), (int)stream.Length);
        }

        /// <summary>
        /// 
        /// </summary>
		/// <param name="message"></param>
        public void SendToAllClients(string message)
        {
            byte[] data = Encoding.ASCII.GetBytes(message);
            SendToAllClients(data, data.Length);
        }

        /// <summary>
        /// 
        /// </summary>
		/// <param name="value"></param>
        public void SendToAllClients(long value)
        {
            lock (this)
            {
                if (mIsShuttingDown == false)
                {
                    foreach (TcpConnection connection in mClientConnections)
                    {
                        try
                        {
                            connection.Writer.Write(value);
                            OnDataSent(new TcpDataSentEventArgs(connection, value));
                        }
                        catch
                        {
                            MarkConnectionForRemoval(connection);
                        }
                    }
                }

                RemoveMarkedConnections();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="data"></param>
        /// <param name="length"></param>
        public void SendToAllClients(byte[] data, int length)
        {
            lock (this)
            {
                if (mIsShuttingDown == false)
                {
                    foreach (TcpConnection connection in mClientConnections)
                    {
                        try
                        {
                            connection.Client.Send(data, length, SocketFlags.None);
                            OnDataSent(new TcpDataSentEventArgs(connection, data));
                        }
                        catch
                        {
                            MarkConnectionForRemoval(connection);
                        }
                    }
                }

                RemoveMarkedConnections();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public void Dispose()
        {
            lock (this)
            {
                if (mIsShuttingDown == false)
                {
                    foreach (TcpConnection connection in mClientConnections)
                    {
                        connection.Dispose();
                    }

                    mClientConnections = null;
                    mConnectionsToRemove = null;
                }
            }
        }

        #region Private Methods

        /// <summary>
        /// Raise the Connected event.
        /// </summary>
        /// <param name="e">An EventArgs object that contains the event data.</param>
        private void OnConnected(TcpConnectedEventArgs e)
        {
            if (Connected != null)
            {
                Connected(this, e);
            }
        }

        /// <summary>
        /// Raise the Disconnected event.
        /// </summary>
        /// <param name="e">An EventArgs object that contains the event data.</param>
        private void OnDisconnected(TcpConnectedEventArgs e)
        {
            if (Disconnected != null)
            {
                Disconnected(this, e);
            }
        }

        /// <summary>
        /// Raise the DataReceived event.
        /// </summary>
        /// <param name="e">An EventArgs object that contains the event data.</param>
        private void OnDataReceived(TcpDataReceivedEventArgs e)
        {
            if (DataReceived != null)
            {
                DataReceived(this, e);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="asyncResult"></param>
        private void OnDataReceived(IAsyncResult asyncResult)
        {
            TcpConnection connection = (TcpConnection)asyncResult.AsyncState;

            lock (this)
            {
                if (mIsShuttingDown == false)
                {
                    try
                    {
                        int bytesReceived = connection.Client.EndReceive(asyncResult);
                        if (bytesReceived > 0)
                        {
                            OnDataReceived(new TcpDataReceivedEventArgs(connection));
                            connection.InitDataReceivedCallback(OnDataReceived);
                        }
                        else
                        {
                            RemoveConnection(connection);
                        }
                    }
                    catch
                    {
                        RemoveConnection(connection);
                    }
                }
            }
        }

        /// <summary>
        /// Raise the DataSent event.
        /// </summary>
        /// <param name="e">An EventArgs object that contains the event data.</param>
        private void OnDataSent(TcpDataSentEventArgs e)
        {
            if (DataSent != null)
            {
                DataSent(this, e);
            }
        }

        /// <summary>
        /// Primary worker thread.
        /// </summary>
        /// <param name="socket"></param>
        private void RunWorker(object socket)
        {
            lock (this)
            {
                if (mIsShuttingDown == false)
                {
                    TcpConnection connection = new TcpConnection((Socket)socket, null);
                    mClientConnections.Add(connection);
                    OnConnected(new TcpConnectedEventArgs(connection));
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="connection"></param>
        private void MarkConnectionForRemoval(TcpConnection connection)
        {
            mConnectionsToRemove.Add(connection);
        }

        /// <summary>
        /// 
        /// </summary>
        private void RemoveMarkedConnections()
        {
            foreach (TcpConnection connection in mConnectionsToRemove)
            {
                RemoveConnection(connection);
            }

            mConnectionsToRemove.Clear();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="connection"></param>
        private void RemoveConnection(TcpConnection connection)
        {
            try
            {
                connection.Dispose();
            }
            catch
            {
                // Igore any exceptions
            }
            finally
            {
                mClientConnections.Remove(connection);
            }

            OnDisconnected(new TcpConnectedEventArgs(connection));
        }

        #endregion

		/// <summary>
		/// 
		/// </summary>
        public static readonly int MaxPendingConnections = 3;

        private IPAddress mIPAddress;
        private int mPort;
        private List<TcpConnection> mClientConnections;
        private List<TcpConnection> mConnectionsToRemove;
        private volatile bool mIsShuttingDown;
        private volatile bool mAcceptingConnections;
    }
}