using System;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;

namespace Bespoke.Common.Net
{
	/// <summary>
	/// 
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	public delegate void UdpDataReceivedHandler(object sender, UdpDataReceivedEventArgs e);

	/// <summary>
	/// 
	/// </summary>
	public class UdpServer
	{
		#region Events

		/// <summary>
		/// 
		/// </summary>
		public event UdpDataReceivedHandler DataReceived;

		#endregion

		#region Properties

		/// <summary>
		/// 
		/// </summary>
		public IPAddress IPAddress
		{
			get
			{
				return mIPAddress;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public int Port
		{
			get
			{
				return mPort;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IPAddress MulticastAddress
		{
			get
			{
				return mMulticastAddress;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool IsRunning
		{
			get
			{
				return mAcceptingConnections;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public TransmissionType TransmissionType
		{
			get
			{
				return mTransmissionType;
			}
		}

		#endregion

		/// <summary>
        /// Binds the server to the loopback address.
        /// </summary>
        /// <param name="port"></param>
        public UdpServer(int port)
            : this(IPAddress.Loopback, port, null, TransmissionType.LocalBroadcast)
        {
        }

		/// <summary>
		/// Binds the server to the loopback address.
		/// </summary>
		/// <param name="port"></param>
		/// <param name="multicastAddress"></param>
		public UdpServer(int port, IPAddress multicastAddress)
			: this(IPAddress.Loopback, port, multicastAddress, TransmissionType.Multicast)
		{
		}

		/// <summary>
        /// 
        /// </summary>
        /// <param name="ipAddress"></param>
        /// <param name="port"></param>
		public UdpServer(IPAddress ipAddress, int port)
			: this(ipAddress, port, null, TransmissionType.Unicast)
		{
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ipAddress"></param>
        /// <param name="port"></param>
		/// <param name="multicastAddress"></param>
		/// <param name="transmissionType"></param>
        public UdpServer(IPAddress ipAddress, int port, IPAddress multicastAddress, TransmissionType transmissionType)
        {
            mPort = port;
            mIPAddress = ipAddress;
			mTransmissionType = transmissionType;

			if (mTransmissionType == TransmissionType.Multicast)
			{
				Trace.Assert(multicastAddress != null);
				mMulticastAddress = multicastAddress;
			}

            mAcceptingConnections = true;
        }

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public static IPAddress[] GetLocalIPAddress()
		{
			IPAddress[] localAddresses = Dns.GetHostAddresses(Dns.GetHostName());
			if (localAddresses.Length == 0)
			{
				throw new Exception("No local IP Address address found.");
			}

			return localAddresses;
		}

		/// <summary>
		/// Start the UDP server and begin receiving data.
		/// </summary>
		public void Start()
		{
			UdpClient udpClient = null;

			try
			{
				switch (mTransmissionType)
				{
					case TransmissionType.Unicast:
						udpClient = new UdpClient(new IPEndPoint(mIPAddress, mPort));
						break;

					case TransmissionType.Multicast:
						udpClient = new UdpClient(mPort);
						udpClient.JoinMulticastGroup(mMulticastAddress);
						break;

					case TransmissionType.Broadcast:
					case TransmissionType.LocalBroadcast:
						udpClient = new UdpClient(mPort);
						break;
				}
				
				while (mAcceptingConnections)
				{
					if (udpClient.Available > 0)
					{
						IPEndPoint remoteEndPoint = null;
						byte[] data = udpClient.Receive(ref remoteEndPoint);
						if (data != null && data.Length > 0)
						{
							OnDataReceived(new UdpDataReceivedEventArgs(remoteEndPoint, data));
						}
					}
				}
			}
			finally
			{
				if (udpClient != null)
				{
					if (mTransmissionType == TransmissionType.Multicast)
					{
						udpClient.DropMulticastGroup(mMulticastAddress);
					}

					udpClient.Close();					
				}
			}
		}

		/// <summary>
		/// Stop the UDP server.
		/// </summary>
		public void Stop()
		{
			mAcceptingConnections = false;
		}

		#region Private Methods

		/// <summary>
		/// Raise the DataReceived event.
		/// </summary>
		/// <param name="e">An EventArgs object that contains the event data.</param>
		private void OnDataReceived(UdpDataReceivedEventArgs e)
		{
			if (DataReceived != null)
			{
				DataReceived(this, e);
			}
		}

		#endregion

		private IPAddress mIPAddress;
		private int mPort;
		private IPAddress mMulticastAddress;
		private TransmissionType mTransmissionType;
		private volatile bool mAcceptingConnections;
	}
}
