﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class AuthorManagerTest : TestFixtureBase
    {
        public static AuthorRecord CreateAuthor()
        {
            AuthorRecord author = new AuthorRecord(SqlDataProvider.DefaultKey, "Test Author", "UCF");
            AuthorManager.AddAuthor(author);
            Trace.WriteLine("CreateAuthor(): Author added.");

            return author;
        }

        public static void DeleteAuthor(AuthorRecord author)
        {
            AuthorManager.DeleteAuthor(author);
            Trace.WriteLine("DeleteAuthor(): Author deleted.");

            AuthorRecord foundAuthor;
            bool found = AuthorManager.RetrieveAuthorByKey(author.Key, out foundAuthor);
            Assert.IsFalse(found, "DeleteAuthor(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mAuthor = CreateAuthor();
        }

        [TearDown]
        public void TearDown()
        {
            DeleteAuthor(mAuthor);
        }

        [Test]
        public void AddAuthor()
        {
            Trace.WriteLine("AddAuthor(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteAuthor()
        {
            AuthorRecord author = new AuthorRecord(SqlDataProvider.DefaultKey, "Delete Author", "UCF");
            AuthorManager.AddAuthor(author);
            Trace.WriteLine("DeleteAuthor(): Author added.");

            AuthorRecord foundAuthor;
            bool found = AuthorManager.RetrieveAuthorByKey(author.Key, out foundAuthor);
            Assert.IsTrue(found, "DeleteAuthor():Pre-delete operation(s) failed.");

            DeleteAuthor(author);
        }

        [Test]
        public void LoadAuthor()
        {
            AuthorRecordList authorList = AuthorManager.LoadAuthor();
            Assert.IsTrue(authorList.Count > 0);

            Trace.WriteLine("LoadAuthor(): " + authorList.Count + " Authors loaded.");
        }

        [Test]
        public void RetrieveAuthorByKey()
        {
            AuthorRecord foundAuthor;
            bool found = AuthorManager.RetrieveAuthorByKey(mAuthor.Key, out foundAuthor);
            Assert.IsTrue(found, "RetrieveAuthorByKey(): Author not found.");
            Assert.IsTrue(mAuthor == foundAuthor);

            Trace.WriteLine("RetrieveAuthorByKey(): Author retrieved.");
        }

        [Test]
        public void RetrieveAuthorByFullName()
        {
            AuthorRecord foundAuthor;
            bool found = AuthorManager.RetrieveAuthorByFullName(mAuthor.FullName, out foundAuthor);
            Assert.IsTrue(found, "RetrieveAuthorByFullName(): Author not found.");
            Assert.IsTrue(mAuthor == foundAuthor);

            Trace.WriteLine("RetrieveAuthorByFullName(): Author retrieved.");
        }

        [Test]
        public void UpdateAuthor()
        {
            AuthorRecord author = new AuthorRecord(SqlDataProvider.DefaultKey, "Update Author", "UCF");
            AuthorManager.AddAuthor(author);
            Trace.WriteLine("UpdateAuthor(): Author added.");

            AuthorRecord foundAuthor;
            bool found = AuthorManager.RetrieveAuthorByKey(author.Key, out foundAuthor);
            Assert.IsTrue(found, "UpdateAuthor(): Pre-update operation(s) failed.");

            author.FullName = "Updated FullName";
            AuthorManager.UpdateAuthor(author);

            found = AuthorManager.RetrieveAuthorByKey(author.Key, out foundAuthor);
            Assert.IsTrue(found, "UpdateAuthor(): Post-update operation(s) failed.");
            Assert.AreEqual(author.FullName, foundAuthor.FullName);

            DeleteAuthor(author);
        }

        private AuthorRecord mAuthor;
    }
}
