﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class DocumentKeywordLinkManagerTest : TestFixtureBase
    {
        public static DocumentKeywordLinkRecord CreateDocumentKeywordLink(DocumentRecord document, KeywordRecord keyword)
        {
            DocumentKeywordLinkRecord dkl = new DocumentKeywordLinkRecord(document.Key, keyword.Key);
            DocumentKeywordLinkManager.AddDocumentKeywordLink(dkl);
            Trace.WriteLine("CreateDocumentKeywordLink(): DocumentKeywordLink added.");

            return dkl;
        }

        public static void DeleteDocumentKeywordLink(DocumentKeywordLinkRecord dkl)
        {
            DocumentKeywordLinkManager.DeleteDocumentKeywordLink(dkl);
            Trace.WriteLine("DeleteDocumentKeywordLink(): DocumentKeywordLink deleted.");

            DocumentKeywordLinkRecord foundDocumentKeywordLink;
            bool found = DocumentKeywordLinkManager.RetrieveDocumentKeywordLinkByKey(dkl.DocumentKey, dkl.KeywordKey, out foundDocumentKeywordLink);
            Assert.IsFalse(found, "DeleteDocumentKeywordLink(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {            
            mUser = UserManagerTest.CreateUser();
            mRepository = RepositoryManagerTest.CreateRepository(mUser);
            mAuthor = AuthorManagerTest.CreateAuthor();
            mPublicationSourceType = PublicationSourceTypeManagerTest.CreatePublicationSourceType();
            mPublicationSource = PublicationSourceManagerTest.CreatePublicationSource(mPublicationSourceType);
            mDocument = DocumentManagerTest.CreateDocument(mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key, mRepository, mAuthor, mPublicationSource);
            mKeyword = KeywordManagerTest.CreateKeyword();

            mDocumentKeywordLink = CreateDocumentKeywordLink(mDocument, mKeyword);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteDocumentKeywordLink(mDocumentKeywordLink);
            DocumentManagerTest.DeleteDocument(mDocument);
            KeywordManagerTest.DeleteKeyword(mKeyword);
            PublicationSourceManagerTest.DeletePublicationSource(mPublicationSource);
            PublicationSourceTypeManagerTest.DeletePublicationSourceType(mPublicationSourceType);
            AuthorManagerTest.DeleteAuthor(mAuthor);
            RepositoryManagerTest.DeleteRepository(mRepository);
            UserManagerTest.DeleteUser(mUser);            
        }

        [Test]
        public void AddDocumentKeywordLink()
        {
            Trace.WriteLine("AddDocumentKeywordLink(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void CreateDocumentKeywordLinkTest()
        {
            DocumentKeywordLinkRecord dkl = new DocumentKeywordLinkRecord(mDocument.Key, mKeyword.Key);
            DocumentKeywordLinkManager.CreateDocumentKeywordLink(dkl);
            Trace.WriteLine("CreateDocumentKeywordLink(): DocumentKeywordLink added.");
        }

        [Test]
        public void DeleteDocumentKeywordLink()
        {
            Trace.WriteLine("AddDocumentKeywordLink(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void LoadDocumentKeywordLinkByDocumentKey()
        {
            DocumentKeywordLinkRecordList dklList = DocumentKeywordLinkManager.LoadDocumentKeywordLinkByDocumentKey(mDocument.Key);
            Assert.IsTrue(dklList.Count > 0);

            Trace.WriteLine("LoadDocumentKeywordLinkByDocumentKey(): " + dklList.Count + " Document-Keyword Link Records loaded.");
        }

        [Test]
        public void LoadDocumentKeywordLinkByKeywordKey()
        {
            DocumentKeywordLinkRecordList dklList = DocumentKeywordLinkManager.LoadDocumentKeywordLinkByKeywordKey(mKeyword.Key);
            Assert.IsTrue(dklList.Count > 0);

            Trace.WriteLine("LoadDocumentKeywordLinkByKeywordKey(): " + dklList.Count + " Document-Keyword Link Records loaded.");
        }

        [Test]
        public void RetrieveDocumentKeywordLinkByKey()
        {
            DocumentKeywordLinkRecord foundDocumentKeywordLink;
            bool found = DocumentKeywordLinkManager.RetrieveDocumentKeywordLinkByKey(mDocument.Key, mKeyword.Key, out foundDocumentKeywordLink);
            Assert.IsTrue(found, "RetrieveDocumentKeywordLinkByKey(): Document-Keyword Link Record not found.");
            Assert.IsTrue(mDocumentKeywordLink == foundDocumentKeywordLink);

            Trace.WriteLine("RetrieveDocumentKeywordLinkByKey(): Document-Keyword Link Record retrieved.");
        }

        private UserRecord mUser;
        private RepositoryRecord mRepository;
        private AuthorRecord mAuthor;
        private PublicationSourceTypeRecord mPublicationSourceType;
        private PublicationSourceRecord mPublicationSource;
        private DocumentRecord mDocument;
        private KeywordRecord mKeyword;
        private DocumentKeywordLinkRecord mDocumentKeywordLink;
    }
}
