﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class DocumentManagerTest : TestFixtureBase
    {
        public static DocumentRecord CreateDocument(int documentTypeKey, RepositoryRecord repository, AuthorRecord author, PublicationSourceRecord publicationSource)
        {
            DocumentRecord document = new DocumentRecord(SqlDataProvider.DefaultKey, documentTypeKey, repository.Key, "Test Document", author.Key, publicationSource.Key, DateTime.Now.Date, "Abstract data", "TestDocument.pdf", Resources.Resources.TestDocument);
            DocumentManager.AddDocument(document);
            Trace.WriteLine("CreateDocument(): Document added.");

            return document;
        }

        public static void DeleteDocument(DocumentRecord document)
        {
            DocumentManager.DeleteDocument(document);
            Trace.WriteLine("DeleteDocument(): Document deleted.");

            DocumentRecord foundDocument;
            bool found = DocumentManager.RetrieveDocumentByKey(document.Key, out foundDocument);
            Assert.IsFalse(found, "DeleteDocument(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mRepository = RepositoryManagerTest.CreateRepository(mUser);
            mAuthor = AuthorManagerTest.CreateAuthor();
            mPublicationSourceType = PublicationSourceTypeManagerTest.CreatePublicationSourceType();
            mPublicationSource = PublicationSourceManagerTest.CreatePublicationSource(mPublicationSourceType);
            mDocument = CreateDocument(mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key, mRepository, mAuthor, mPublicationSource);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteDocument(mDocument);
            PublicationSourceManagerTest.DeletePublicationSource(mPublicationSource);
            PublicationSourceTypeManagerTest.DeletePublicationSourceType(mPublicationSourceType);
            AuthorManagerTest.DeleteAuthor(mAuthor);
            RepositoryManagerTest.DeleteRepository(mRepository);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddDocument()
        {
            Trace.WriteLine("AddDocument(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteDocument()
        {
            Trace.WriteLine("DeleteDocument(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void GetDocumentCount()
        {
            int documentCount = DocumentManager.GetDocumentCount(mRepository.Key);

            Trace.WriteLine("GetDocumentCount(): " + documentCount);
        }

        [Test]
        public void LoadDocument()
        {
            DocumentRecordList documentList = DocumentManager.LoadDocument(mRepository.Key);
            Assert.IsTrue(documentList.Count > 0);

            Trace.WriteLine("LoadDocument(): " + documentList.Count + " Documents loaded.");
        }

        [Test]
        public void LoadDocumentByAuthorKey()
        {
            DocumentRecordList documentList = DocumentManager.LoadDocumentByAuthorKey(mRepository.Key, mAuthor.Key);
            Assert.IsTrue(documentList.Count > 0);

            Trace.WriteLine("LoadDocument(): " + documentList.Count + " Documents loaded.");
        }

        [Test]
        public void LoadDocumentByDocumentTypeKey()
        {
            DocumentRecordList documentList = DocumentManager.LoadDocumentByDocumentTypeKey(mRepository.Key, mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key);
            Assert.IsTrue(documentList.Count > 0);

            Trace.WriteLine("LoadDocument(): " + documentList.Count + " Documents loaded.");
        }

        [Test]
        public void LoadDocumentByPublicationSourceKey()
        {
            DocumentRecordList documentList = DocumentManager.LoadDocumentByPublicationSourceKey(mRepository.Key, mPublicationSource.Key);
            Assert.IsTrue(documentList.Count > 0);

            Trace.WriteLine("LoadDocument(): " + documentList.Count + " Documents loaded.");
        }

        [Test]
        public void RetrieveDocumentByKey()
        {
            DocumentRecord foundDocument;
            bool found = DocumentManager.RetrieveDocumentByKey(mDocument.Key, out foundDocument);
            Assert.IsTrue(found, "RetrieveDocumentByKey(): Document not found.");
            Assert.IsTrue(mDocument == foundDocument);

            Trace.WriteLine("RetrieveDocumentByKey(): Document retrieved.");
        }

        [Test]
        public void UpdateDocument()
        {
            DocumentRecord document = new DocumentRecord(SqlDataProvider.DefaultKey, mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key, mRepository.Key, "Update Document", mAuthor.Key, mPublicationSource.Key, DateTime.Now.Date, "Abstract data", "TestDocument.pdf", Resources.Resources.TestDocument);
            DocumentManager.AddDocument(document);
            Trace.WriteLine("UpdateDocument(): Document added.");

            DocumentRecord foundDocument;
            bool found = DocumentManager.RetrieveDocumentByKey(document.Key, out foundDocument);
            Assert.IsTrue(found, "UpdateDocument(): Pre-update operation(s) failed.");

            document.Title = "Updated Document Title";
            DocumentManager.UpdateDocument(document);

            found = DocumentManager.RetrieveDocumentByKey(document.Key, out foundDocument);
            Assert.IsTrue(found, "UpdateDocument(): Post-update operation(s) failed.");
            Assert.AreEqual(document.Title, foundDocument.Title);

            DeleteDocument(document);
        }

        private UserRecord mUser;
        private RepositoryRecord mRepository;
        private AuthorRecord mAuthor;
        private PublicationSourceTypeRecord mPublicationSourceType;
        private PublicationSourceRecord mPublicationSource;
        private DocumentRecord mDocument;
    }
}