﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class GroupManagerTest : TestFixtureBase
    {
        public static GroupRecord CreateGroup(UserRecord user)
        {
            GroupRecord group = new GroupRecord(SqlDataProvider.DefaultKey, user.Key, "Test Group");
            GroupManager.AddGroup(group);
            Trace.WriteLine("CreateGroup(): Group added.");

            return group;
        }

        public static void DeleteGroup(GroupRecord group)
        {
            GroupManager.DeleteGroup(group);
            Trace.WriteLine("DeleteGroup(): Group deleted.");

            GroupRecord foundGroup;
            bool found = GroupManager.RetrieveGroupByKey(group.Key, out foundGroup);
            Assert.IsFalse(found, "DeleteGroup(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mGroup = CreateGroup(mUser);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteGroup(mGroup);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddGroup()
        {
            Trace.WriteLine("AddGroup(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteGroup()
        {
            GroupRecord group = new GroupRecord(SqlDataProvider.DefaultKey, mUser.Key, "Delete Group");
            GroupManager.AddGroup(group);
            Trace.WriteLine("DeleteGroup(): Group added.");

            GroupRecord foundGroup;
            bool found = GroupManager.RetrieveGroupByKey(group.Key, out foundGroup);
            Assert.IsTrue(found, "DeleteGroup():Pre-delete operation(s) failed.");

            DeleteGroup(group);
        }

        [Test]
        public void LoadGroup()
        {
            GroupRecordList groupList = GroupManager.LoadGroup();
            Assert.IsTrue(groupList.Count > 0);

            Trace.WriteLine("LoadGroup(): " + groupList.Count + " Groups loaded.");
        }

        [Test]
        public void LoadGroupByUserKey()
        {
            GroupRecordList groupList = GroupManager.LoadGroupByUserKey(mUser.Key);
            Assert.IsTrue(groupList.Count > 0);

            Trace.WriteLine("LoadGroup(): " + groupList.Count + " Groups loaded.");
        }

        [Test]
        public void RetrieveGroupByKey()
        {
            GroupRecord foundGroup;
            bool found = GroupManager.RetrieveGroupByKey(mGroup.Key, out foundGroup);
            Assert.IsTrue(found, "RetrieveGroupByKey(): Group not found.");
            Assert.IsTrue(mGroup == foundGroup);

            Trace.WriteLine("RetrieveGroupByKey(): Group retrieved.");
        }

        [Test]
        public void RetrieveGroupByName()
        {
            GroupRecord foundGroup;
            bool found = GroupManager.RetrieveGroupByName(mGroup.UserKey, mGroup.Name, out foundGroup);
            Assert.IsTrue(found, "RetrieveGroupByKey(): Group not found.");
            Assert.IsTrue(mGroup == foundGroup);

            Trace.WriteLine("RetrieveGroupByKey(): Group retrieved.");
        }

        [Test]
        public void UpdateGroup()
        {
            GroupRecord group = new GroupRecord(SqlDataProvider.DefaultKey, mUser.Key, "Update Group");
            GroupManager.AddGroup(group);
            Trace.WriteLine("UpdateGroup(): Group added.");

            GroupRecord foundGroup;
            bool found = GroupManager.RetrieveGroupByKey(group.Key, out foundGroup);
            Assert.IsTrue(found, "UpdateGroup(): Pre-update operation(s) failed.");

            group.Name = "Updated Group Name";
            GroupManager.UpdateGroup(group);

            found = GroupManager.RetrieveGroupByKey(group.Key, out foundGroup);
            Assert.IsTrue(found, "UpdateGroup(): Post-update operation(s) failed.");
            Assert.AreEqual(group.Name, foundGroup.Name);

            DeleteGroup(group);
        }

        private UserRecord mUser;
        private GroupRecord mGroup;
    }
}