﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class KeywordManagerTest : TestFixtureBase
    {
        public static KeywordRecord CreateKeyword()
        {
            KeywordRecord keyword = new KeywordRecord(SqlDataProvider.DefaultKey, "Testing");
            KeywordManager.AddKeyword(keyword);
            Trace.WriteLine("CreateKeyword(): Keyword added.");

            return keyword;
        }

        public static void DeleteKeyword(KeywordRecord keyword)
        {
            KeywordManager.DeleteKeyword(keyword);
            Trace.WriteLine("DeleteKeyword(): Keyword deleted.");

            KeywordRecord foundKeyword;
            bool found = KeywordManager.RetrieveKeywordByKey(keyword.Key, out foundKeyword);
            Assert.IsFalse(found, "DeleteKeyword(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mKeyword = CreateKeyword();
        }

        [TearDown]
        public void TearDown()
        {
            DeleteKeyword(mKeyword);
        }

        [Test]
        public void AddKeyword()
        {
            Trace.WriteLine("AddKeyword(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void CreateKeywordTest()
        {
            KeywordRecord keyword = new KeywordRecord(SqlDataProvider.DefaultKey, "TestingCreate");
            KeywordManager.CreateKeyword(keyword);
            Trace.WriteLine("CreateKeyword(): Keyword added.");

            KeywordRecord duplicateKeyword = new KeywordRecord(SqlDataProvider.DefaultKey, "TestingCreate");
            KeywordManager.CreateKeyword(duplicateKeyword);
            Assert.AreEqual(keyword.Word, duplicateKeyword.Word);

            DeleteKeyword(keyword);
        }

        [Test]
        public void DeleteKeyword()
        {
            KeywordRecord keyword = new KeywordRecord(SqlDataProvider.DefaultKey, "Delete");
            KeywordManager.AddKeyword(keyword);
            Trace.WriteLine("DeleteKeyword(): Keyword added.");

            KeywordRecord foundKeyword;
            bool found = KeywordManager.RetrieveKeywordByKey(keyword.Key, out foundKeyword);
            Assert.IsTrue(found, "DeleteKeyword():Pre-delete operation(s) failed.");

            DeleteKeyword(keyword);
        }

        [Test]
        public void LoadKeyword()
        {
            KeywordRecordList keywordList = KeywordManager.LoadKeyword();
            Assert.IsTrue(keywordList.Count > 0);

            Trace.WriteLine("LoadKeyword(): " + keywordList.Count + " Keywords loaded.");
        }

        [Test]
        public void RetrieveKeywordByKey()
        {
            KeywordRecord foundKeyword;
            bool found = KeywordManager.RetrieveKeywordByKey(mKeyword.Key, out foundKeyword);
            Assert.IsTrue(found, "RetrieveKeywordByKey(): Keyword not found.");
            Assert.IsTrue(mKeyword == foundKeyword);

            Trace.WriteLine("RetrieveKeywordByKey(): Keyword retrieved.");
        }

        [Test]
        public void RetrieveKeywordByWord()
        {
            KeywordRecord foundKeyword;
            bool found = KeywordManager.RetrieveKeywordByWord(mKeyword.Word, out foundKeyword);
            Assert.IsTrue(found, "RetrieveKeywordByKey(): Keyword not found.");
            Assert.IsTrue(mKeyword == foundKeyword);

            Trace.WriteLine("RetrieveKeywordByKey(): Keyword retrieved.");
        }

        [Test]
        public void UpdateKeyword()
        {
            KeywordRecord keyword = new KeywordRecord(SqlDataProvider.DefaultKey, "Update");
            KeywordManager.AddKeyword(keyword);
            Trace.WriteLine("UpdateKeyword(): Keyword added.");

            KeywordRecord foundKeyword;
            bool found = KeywordManager.RetrieveKeywordByKey(keyword.Key, out foundKeyword);
            Assert.IsTrue(found, "UpdateKeyword(): Pre-update operation(s) failed.");

            keyword.Word = "Updated";
            KeywordManager.UpdateKeyword(keyword);

            found = KeywordManager.RetrieveKeywordByKey(keyword.Key, out foundKeyword);
            Assert.IsTrue(found, "UpdateKeyword(): Post-update operation(s) failed.");
            Assert.AreEqual(keyword.Word, foundKeyword.Word);

            DeleteKeyword(keyword);
        }

        private KeywordRecord mKeyword;
    }
}