﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class PublicationSourceManagerTest : TestFixtureBase
    {
        public static PublicationSourceRecord CreatePublicationSource(PublicationSourceTypeRecord publicationSourceType)
        {
            PublicationSourceRecord publicationSource = new PublicationSourceRecord(SqlDataProvider.DefaultKey, "Test PublicationSource", publicationSourceType.Key);
            PublicationSourceManager.AddPublicationSource(publicationSource);
            Trace.WriteLine("CreatePublicationSource(): PublicationSource added.");

            return publicationSource;
        }

        public static void DeletePublicationSource(PublicationSourceRecord publicationSource)
        {
            PublicationSourceManager.DeletePublicationSource(publicationSource);
            Trace.WriteLine("DeletePublicationSource(): PublicationSource deleted.");

            PublicationSourceRecord foundPublicationSource;
            bool found = PublicationSourceManager.RetrievePublicationSourceByKey(publicationSource.Key, out foundPublicationSource);
            Assert.IsFalse(found, "DeletePublicationSource(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mPublicationSourceType = PublicationSourceTypeManagerTest.CreatePublicationSourceType();
            mPublicationSource = CreatePublicationSource(mPublicationSourceType);
        }

        [TearDown]
        public void TearDown()
        {
            DeletePublicationSource(mPublicationSource);
            PublicationSourceTypeManagerTest.DeletePublicationSourceType(mPublicationSourceType);
        }

        [Test]
        public void AddPublicationSource()
        {
            Trace.WriteLine("AddPublicationSource(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeletePublicationSource()
        {
            PublicationSourceRecord publicationSource = new PublicationSourceRecord(SqlDataProvider.DefaultKey, "Delete PublicationSource", mPublicationSourceType.Key);
            PublicationSourceManager.AddPublicationSource(publicationSource);
            Trace.WriteLine("DeletePublicationSource(): PublicationSource added.");

            PublicationSourceRecord foundPublicationSource;
            bool found = PublicationSourceManager.RetrievePublicationSourceByKey(publicationSource.Key, out foundPublicationSource);
            Assert.IsTrue(found, "DeletePublicationSource():Pre-delete operation(s) failed.");

            DeletePublicationSource(publicationSource);
        }

        [Test]
        public void LoadPublicationSource()
        {
            PublicationSourceRecordList publicationSourceList = PublicationSourceManager.LoadPublicationSource();
            Assert.IsTrue(publicationSourceList.Count > 0);

            Trace.WriteLine("LoadPublicationSource(): " + publicationSourceList.Count + " PublicationSources loaded.");
        }

        [Test]
        public void LoadPublicationSourceByTypeKey()
        {
            PublicationSourceRecordList publicationSourceList = PublicationSourceManager.LoadPublicationSourceByTypeKey(mPublicationSourceType.Key);
            Assert.IsTrue(publicationSourceList.Count > 0);

            Trace.WriteLine("LoadPublicationSourceByTypeKey(): " + publicationSourceList.Count + " PublicationSources loaded.");
        }

        [Test]
        public void RetrievePublicationSourceByKey()
        {
            PublicationSourceRecord foundPublicationSource;
            bool found = PublicationSourceManager.RetrievePublicationSourceByKey(mPublicationSource.Key, out foundPublicationSource);
            Assert.IsTrue(found, "RetrievePublicationSourceByKey(): PublicationSource not found.");
            Assert.IsTrue(mPublicationSource == foundPublicationSource);

            Trace.WriteLine("RetrievePublicationSourceByKey(): PublicationSource retrieved.");
        }

        [Test]
        public void UpdatePublicationSource()
        {
            PublicationSourceRecord publicationSource = new PublicationSourceRecord(SqlDataProvider.DefaultKey, "Update PublicationSource", mPublicationSourceType.Key);
            PublicationSourceManager.AddPublicationSource(publicationSource);
            Trace.WriteLine("UpdatePublicationSource(): PublicationSource added.");

            PublicationSourceRecord foundPublicationSource;
            bool found = PublicationSourceManager.RetrievePublicationSourceByKey(publicationSource.Key, out foundPublicationSource);
            Assert.IsTrue(found, "UpdatePublicationSource(): Pre-update operation(s) failed.");

            publicationSource.Name = "Updated PublicationSource Name";
            PublicationSourceManager.UpdatePublicationSource(publicationSource);

            found = PublicationSourceManager.RetrievePublicationSourceByKey(publicationSource.Key, out foundPublicationSource);
            Assert.IsTrue(found, "UpdatePublicationSource(): Post-update operation(s) failed.");
            Assert.AreEqual(publicationSource.Name, foundPublicationSource.Name);

            DeletePublicationSource(publicationSource);
        }


        private PublicationSourceTypeRecord mPublicationSourceType;
        private PublicationSourceRecord mPublicationSource;
    }
}