﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class RepositoryManagerTest : TestFixtureBase
    {
        public static RepositoryRecord CreateRepository(UserRecord user)
        {
            RepositoryRecord repository = new RepositoryRecord(SqlDataProvider.DefaultKey, user.Key, default(DateTime), "Test Repository", "This is a test repository.");
            RepositoryManager.AddRepository(repository);
            Trace.WriteLine("CreateRepository(): Repository added.");

            return repository;
        }

        public static void DeleteRepository(RepositoryRecord repository)
        {
            RepositoryManager.DeleteRepository(repository);
            Trace.WriteLine("DeleteRepository(): Repository deleted.");

            RepositoryRecord foundRepository;
            bool found = RepositoryManager.RetrieveRepositoryByKey(repository.Key, out foundRepository);
            Assert.IsFalse(found, "DeleteRepository(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mRepository = CreateRepository(mUser);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteRepository(mRepository);            
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddRepository()
        {
            Trace.WriteLine("AddRepository(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteRepository()
        {
            RepositoryRecord repository = new RepositoryRecord(SqlDataProvider.DefaultKey, mUser.Key, DateTime.Now, "Delete Repository", "This is a test repository.");
            RepositoryManager.AddRepository(repository);
            Trace.WriteLine("DeleteRepository(): Repository added.");

            RepositoryRecord foundRepository;
            bool found = RepositoryManager.RetrieveRepositoryByKey(repository.Key, out foundRepository);
            Assert.IsTrue(found, "DeleteRepository():Pre-delete operation(s) failed.");

            DeleteRepository(repository);
        }

        [Test]
        public void LoadRepository()
        {
            RepositoryRecordList repositoryList = RepositoryManager.LoadRepository();
            Assert.IsTrue(repositoryList.Count > 0);

            Trace.WriteLine("LoadRepository(): " + repositoryList.Count + " Repositories loaded.");
        }

        [Test]
        public void LoadRepositoryByUserKey()
        {
            RepositoryRecordList repositoryList = RepositoryManager.LoadRepositoryByUserKey(mUser.Key);
            Assert.IsTrue(repositoryList.Count > 0);

            Trace.WriteLine("LoadRepository(): " + repositoryList.Count + " Repositories loaded.");
        }

        [Test]
        public void RetrieveRepositoryByKey()
        {
            RepositoryRecord foundRepository;
            bool found = RepositoryManager.RetrieveRepositoryByKey(mRepository.Key, out foundRepository);
            Assert.IsTrue(found, "RetrieveRepositoryByKey(): Repository not found.");
            Assert.IsTrue(mRepository == foundRepository);

            Trace.WriteLine("RetrieveRepositoryByKey(): Repository retrieved.");
        }

        [Test]
        public void RetrieveRepositoryByName()
        {
            RepositoryRecord foundRepository;
            bool found = RepositoryManager.RetrieveRepositoryByName(mUser.Key, mRepository.Name, out foundRepository);
            Assert.IsTrue(found, "RetrieveRepositoryByKey(): Repository not found.");
            Assert.IsTrue(mRepository == foundRepository);

            Trace.WriteLine("RetrieveRepositoryByKey(): Repository retrieved.");
        }

        [Test]
        public void UpdateRepository()
        {
            RepositoryRecord repository = new RepositoryRecord(SqlDataProvider.DefaultKey, mUser.Key, DateTime.Now, "Update Repository", "This is a test repository.");
            RepositoryManager.AddRepository(repository);
            Trace.WriteLine("UpdateRepository(): Repository added.");

            RepositoryRecord foundRepository;
            bool found = RepositoryManager.RetrieveRepositoryByKey(repository.Key, out foundRepository);
            Assert.IsTrue(found, "UpdateRepository(): Pre-update operation(s) failed.");

            repository.Name = "Updated Repository Name";
            RepositoryManager.UpdateRepository(repository);

            found = RepositoryManager.RetrieveRepositoryByKey(repository.Key, out foundRepository);
            Assert.IsTrue(found, "UpdateRepository(): Post-update operation(s) failed.");
            Assert.AreEqual(repository.Name, foundRepository.Name);

            DeleteRepository(repository);
        }


        private UserRecord mUser;
        private RepositoryRecord mRepository;
    }
}
