﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class ReviewManagerTest : TestFixtureBase
    {
        public static ReviewRecord CreateReview(DocumentRecord document, UserRecord user)
        {
            ReviewRecord review = new ReviewRecord(SqlDataProvider.DefaultKey, document.Key, default(DateTime), user.Key, 0, "Test Comments", Resources.Resources.TestDocument);
            ReviewManager.AddReview(review);
            Trace.WriteLine("CreateReview(): Review added.");

            return review;
        }

        public static void DeleteReview(ReviewRecord review)
        {
            ReviewManager.DeleteReview(review);
            Trace.WriteLine("DeleteReview(): Review deleted.");

            ReviewRecord foundReview;
            bool found = ReviewManager.RetrieveReviewByKey(review.Key, out foundReview);
            Assert.IsFalse(found, "DeleteReview(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mRepository = RepositoryManagerTest.CreateRepository(mUser);
            mAuthor = AuthorManagerTest.CreateAuthor();
            mPublicationSourceType = PublicationSourceTypeManagerTest.CreatePublicationSourceType();
            mPublicationSource = PublicationSourceManagerTest.CreatePublicationSource(mPublicationSourceType);
            mDocument = DocumentManagerTest.CreateDocument(mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key, mRepository, mAuthor, mPublicationSource);
            mReview = CreateReview(mDocument, mUser);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteReview(mReview);
            DocumentManagerTest.DeleteDocument(mDocument);
            PublicationSourceManagerTest.DeletePublicationSource(mPublicationSource);
            PublicationSourceTypeManagerTest.DeletePublicationSourceType(mPublicationSourceType);
            AuthorManagerTest.DeleteAuthor(mAuthor);
            RepositoryManagerTest.DeleteRepository(mRepository);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddReview()
        {
            Trace.WriteLine("AddReview(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteReview()
        {
            Trace.WriteLine("DeleteReview(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void LoadReview()
        {
            ReviewRecordList reviewList = ReviewManager.LoadReview(mDocument.Key);
            Assert.IsTrue(reviewList.Count > 0);

            Trace.WriteLine("LoadReview(): " + reviewList.Count + " Reviews loaded.");
        }

        [Test]
        public void RetrieveReviewByKey()
        {
            ReviewRecord foundReview;
            bool found = ReviewManager.RetrieveReviewByKey(mReview.Key, out foundReview);
            Assert.IsTrue(found, "RetrieveReviewByKey(): Review not found.");
            Assert.IsTrue(mReview == foundReview);

            Trace.WriteLine("RetrieveReviewByKey(): Review retrieved.");
        }

        [Test]
        public void UpdateReview()
        {
            ReviewRecord review = new ReviewRecord(SqlDataProvider.DefaultKey, mDocument.Key, default(DateTime), mUser.Key, 0, "Test Comments for Update", Resources.Resources.TestDocument);
            ReviewManager.AddReview(review);
            Trace.WriteLine("UpdateReview(): Review added.");

            ReviewRecord foundReview;
            bool found = ReviewManager.RetrieveReviewByKey(review.Key, out foundReview);
            Assert.IsTrue(found, "UpdateReview(): Pre-update operation(s) failed.");

            review.Comments = "Updated Review Comments";
            ReviewManager.UpdateReview(review);

            found = ReviewManager.RetrieveReviewByKey(review.Key, out foundReview);
            Assert.IsTrue(found, "UpdateReview(): Post-update operation(s) failed.");
            Assert.AreEqual(review.Comments, foundReview.Comments);

            DeleteReview(review);
        }

        private UserRecord mUser;
        private RepositoryRecord mRepository;
        private AuthorRecord mAuthor;
        private PublicationSourceTypeRecord mPublicationSourceType;
        private PublicationSourceRecord mPublicationSource;
        private DocumentRecord mDocument;
        private ReviewRecord mReview;
    }
}