﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class UserManagerTest : TestFixtureBase
    {
        public static UserRecord CreateUser()
        {
            UserRecord user = new UserRecord(SqlDataProvider.DefaultKey, "testuser", "Pass-Word", true, "Test User", "testuser@somewhere.com", "http://www.somewhere.com/");
            UserManager.AddUser(user);
            Trace.WriteLine("CreateUser(): User added.");

            return user;
        }

        public static void DeleteUser(UserRecord user)
        {
            UserManager.DeleteUser(user);
            Trace.WriteLine("DeleteUser(): User deleted.");

            UserRecord foundUser;
            bool found = UserManager.RetrieveUserByKey(user.Key, out foundUser);
            Assert.IsFalse(found, "DeleteUser(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = CreateUser();
        }

        [TearDown]
        public void TearDown()
        {
            DeleteUser(mUser);
        }

        [Test]
        public void AddUser()
        {
            Trace.WriteLine("AddUser(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteUser()
        {
            UserRecord user = new UserRecord(SqlDataProvider.DefaultKey, "deleteuser", "Pass-Word", false, "Test User", "testuser@somewhere.com", "http://www.somewhere.com/");
            UserManager.AddUser(user);
            Trace.WriteLine("DeleteUser(): User added.");

            UserRecord foundUser;
            bool found = UserManager.RetrieveUserByKey(user.Key, out foundUser);
            Assert.IsTrue(found, "DeleteUser():Pre-delete operation(s) failed.");

            DeleteUser(user);
        }

        [Test]
        public void LoadUser()
        {
            UserRecordList userList = UserManager.LoadUser();
            Assert.IsTrue(userList.Count > 0);

            Trace.WriteLine("LoadUser(): " + userList.Count + " Users loaded.");
        }

        [Test]
        public void RetrieveUserByKey()
        {
            UserRecord foundUser;
            bool found = UserManager.RetrieveUserByKey(mUser.Key, out foundUser);
            Assert.IsTrue(found, "RetrieveUserByKey(): User not found.");
            Assert.IsTrue(mUser == foundUser);

            Trace.WriteLine("RetrieveUserByKey(): User retrieved.");
        }

        [Test]
        public void RetrieveUserByUsername()
        {
            UserRecord foundUser;
            bool found = UserManager.RetrieveUserByUsername(mUser.Username, out foundUser);
            Assert.IsTrue(found, "RetrieveUserByUsername(): User not found.");
            Assert.IsTrue(mUser == foundUser);

            Trace.WriteLine("RetrieveUserByUsername(): User retrieved.");
        }

        [Test]
        public void UpdateUser()
        {
            UserRecord user = new UserRecord(SqlDataProvider.DefaultKey, "updateuser", "Pass-Word", false, "Test User", "testuser@somewhere.com", "http://www.somewhere.com/");
            UserManager.AddUser(user);
            Trace.WriteLine("UpdateUser(): User added.");

            UserRecord foundUser;
            bool found = UserManager.RetrieveUserByKey(user.Key, out foundUser);
            Assert.IsTrue(found, "UpdateUser(): Pre-update operation(s) failed.");

            user.FullName = "Updated FullName";
            UserManager.UpdateUser(user);

            found = UserManager.RetrieveUserByKey(user.Key, out foundUser);
            Assert.IsTrue(found, "UpdateUser(): Post-update operation(s) failed.");
            Assert.AreEqual(user.FullName, foundUser.FullName);

            DeleteUser(user);
        }

        private UserRecord mUser;
    }
}
