﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class AuthorManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the author.
        /// </summary>
        /// <param name="author">The author.</param>
        public static void AddAuthor(AuthorRecord author)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddAuthor";
                    sDataProvider.AddParameter("@FullName", SqlDbType.VarChar, ParameterDirection.Input, AuthorRecord.FullNameSize, author.FullName);
                    sDataProvider.AddParameter("@Affiliation", SqlDbType.VarChar, ParameterDirection.Input, AuthorRecord.AffiliationSize, author.Affiliation);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    author.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the author.
        /// </summary>
        /// <param name="author">The author.</param>
        /// <returns></returns>
        public static bool DeleteAuthor(AuthorRecord author)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteAuthor";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, author.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all authors in the database.
        /// </summary>
        /// <returns>The retrieved author collection.</returns>
        public static AuthorRecordList LoadAuthor()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadAuthor";

                return RetrieveMultipleAuthors();
            }
        }

        /// <summary>
        /// Retrieves the author by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="author">The author.</param>
        /// <returns></returns>
        public static bool RetrieveAuthorByKey(int key, out AuthorRecord author)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveAuthorByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, AuthorRecord.KeySize, key);

                return RetrieveAuthor(out author);
            }
        }

        /// <summary>
        /// Retrieves the author by name.
        /// </summary>
        /// <param name="fullName">The full name.</param>
        /// <param name="author">The author.</param>
        /// <returns></returns>
        public static bool RetrieveAuthorByFullName(string fullName, out AuthorRecord author)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveAuthorByFullName";
                sDataProvider.AddParameter("@FullName", SqlDbType.VarChar, ParameterDirection.Input, AuthorRecord.FullNameSize, fullName);

                return RetrieveAuthor(out author);
            }
        }

        /// <summary>
        /// Updates the author.
        /// </summary>
        /// <param name="author">The author.</param>
        /// <returns></returns>
        public static bool UpdateAuthor(AuthorRecord author)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateAuthor";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, AuthorRecord.KeySize, author.Key);
                    sDataProvider.AddParameter("@FullName", SqlDbType.VarChar, ParameterDirection.Input, AuthorRecord.FullNameSize, author.FullName);
                    sDataProvider.AddParameter("@Affiliation", SqlDbType.VarChar, ParameterDirection.Input, AuthorRecord.AffiliationSize, author.Affiliation);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the author.
        /// </summary>
        /// <param name="author">The author.</param>
        /// <returns></returns>
        private static bool RetrieveAuthor(out AuthorRecord author)
        {
            try
            {
                author = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        author = new AuthorRecord();
                        author.Key = sDataProvider.ReadValue<int>("Key");
                        author.FullName = sDataProvider.ReadValue<string>("FullName");
                        author.Affiliation = sDataProvider.ReadValue<string>("Affiliation");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the authors.
        /// </summary>
        /// <returns></returns>
        private static AuthorRecordList RetrieveMultipleAuthors()
        {
            AuthorRecordList authorList = new AuthorRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        AuthorRecord author = new AuthorRecord();
                        author.Key = sDataProvider.ReadValue<int>("Key");
                        author.FullName = sDataProvider.ReadValue<string>("FullName");
                        author.Affiliation = sDataProvider.ReadValue<string>("Affiliation");

                        authorList.Add(author);
                    }

                    return authorList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}