﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class DocumentManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the document.
        /// </summary>
        /// <param name="document">The document.</param>
        public static void AddDocument(DocumentRecord document)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddDocument";
                    sDataProvider.AddParameter("@DocumentTypeKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, document.DocumentTypeKey);
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, document.RepositoryKey);
                    sDataProvider.AddParameter("@Title", SqlDbType.VarChar, ParameterDirection.Input, DocumentRecord.TitleSize, document.Title);
                    sDataProvider.AddParameter("@AuthorKey", SqlDbType.Int, ParameterDirection.Input, AuthorRecord.KeySize, document.AuthorKey);
                    sDataProvider.AddParameter("@PublicationSourceKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceRecord.KeySize, document.PublicationSourceKey);
                    sDataProvider.AddParameter("@PublicationDate", SqlDbType.DateTime, ParameterDirection.Input, DocumentRecord.PublicationDateSize, document.PublicationDate);
                    sDataProvider.AddParameter("@Abstract", SqlDbType.Text, ParameterDirection.Input, DocumentRecord.AbstractSize, document.Abstract);
                    sDataProvider.AddParameter("@FileName", SqlDbType.VarChar, ParameterDirection.Input, DocumentRecord.FileNameSize, document.FileName);
                    sDataProvider.AddParameter("@Data", SqlDbType.Image, ParameterDirection.Input, DocumentRecord.DataSize, document.Data);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    document.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        public static bool DeleteDocument(DocumentRecord document)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteDocument";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, document.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Gets the document count.
        /// </summary>
        /// <param name="repositoryKey">The repository key.</param>
        /// <returns></returns>
        public static int GetDocumentCount(int repositoryKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "GetDocumentCount";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="repositoryKey">The repository key.</param>
        /// <returns>The retrieved document collection.</returns>
        public static DocumentRecordList LoadDocument(int repositoryKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocument";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="repositoryKey">The repository key.</param>
        /// <param name="documentTypeKey">The document type key.</param>
        /// <returns>The retrieved document collection.</returns>
        public static DocumentRecordList LoadDocumentByDocumentTypeKey(int repositoryKey, int documentTypeKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentByDocumentTypeKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);
                sDataProvider.AddParameter("@DocumentTypeKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, documentTypeKey);

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="repositoryKey">The repository key.</param>
        /// <param name="authorKey">The author key.</param>
        /// <returns>The retrieved document collection.</returns>
        public static DocumentRecordList LoadDocumentByAuthorKey(int repositoryKey, int authorKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentByAuthorKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);
                sDataProvider.AddParameter("@AuthorKey", SqlDbType.Int, ParameterDirection.Input, AuthorRecord.KeySize, authorKey);

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="repositoryKey">The repository key.</param>        
        /// <returns>The retrieved document collection.</returns>
        public static DocumentRecordList LoadDocumentByPublicationSourceKey(int repositoryKey, int publicationSourceKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentByPublicationSourceKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);
                sDataProvider.AddParameter("@PublicationSourceKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceRecord.KeySize, publicationSourceKey);

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieves the document by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        public static bool RetrieveDocumentByKey(int key, out DocumentRecord document)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveDocumentByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, key);

                return RetrieveDocument(out document);
            }
        }

        /// <summary>
        /// Updates the document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        public static bool UpdateDocument(DocumentRecord document)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateDocument";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, document.Key);
                    sDataProvider.AddParameter("@DocumentTypeKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, document.DocumentTypeKey);
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, document.RepositoryKey);
                    sDataProvider.AddParameter("@Title", SqlDbType.VarChar, ParameterDirection.Input, DocumentRecord.TitleSize, document.Title);
                    sDataProvider.AddParameter("@AuthorKey", SqlDbType.Int, ParameterDirection.Input, AuthorRecord.KeySize, document.AuthorKey);
                    sDataProvider.AddParameter("@PublicationSourceKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceRecord.KeySize, document.PublicationSourceKey);
                    sDataProvider.AddParameter("@PublicationDate", SqlDbType.DateTime, ParameterDirection.Input, DocumentRecord.PublicationDateSize, document.PublicationDate);
                    sDataProvider.AddParameter("@Abstract", SqlDbType.Text, ParameterDirection.Input, DocumentRecord.AbstractSize, document.Abstract);
                    sDataProvider.AddParameter("@FileName", SqlDbType.VarChar, ParameterDirection.Input, DocumentRecord.FileNameSize, document.FileName);
                    sDataProvider.AddParameter("@Data", SqlDbType.Image, ParameterDirection.Input, DocumentRecord.DataSize, document.Data);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        private static bool RetrieveDocument(out DocumentRecord document)
        {
            try
            {
                document = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        document = new DocumentRecord();
                        document.Key = sDataProvider.ReadValue<int>("Key");
                        document.DocumentTypeKey = sDataProvider.ReadValue<int>("DocumentTypeKey");
                        document.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        document.Title = sDataProvider.ReadValue<string>("Title");
                        document.AuthorKey = sDataProvider.ReadValue<int>("AuthorKey");
                        document.PublicationSourceKey = sDataProvider.ReadValue<int>("PublicationSourceKey");
                        document.PublicationDate = sDataProvider.ReadValue<DateTime>("PublicationDate");
                        document.Abstract = sDataProvider.ReadValue<string>("Abstract");
                        document.FileName = sDataProvider.ReadValue<string>("FileName");
                        document.Data = sDataProvider.ReadValue<byte[]>("Data");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the repositories.
        /// </summary>
        /// <returns></returns>
        private static DocumentRecordList RetrieveMultipleRepositories()
        {
            DocumentRecordList documentList = new DocumentRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        DocumentRecord document = new DocumentRecord();
                        document.Key = sDataProvider.ReadValue<int>("Key");
                        document.DocumentTypeKey = sDataProvider.ReadValue<int>("DocumentTypeKey");
                        document.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        document.Title = sDataProvider.ReadValue<string>("Title");
                        document.AuthorKey = sDataProvider.ReadValue<int>("AuthorKey");
                        document.PublicationSourceKey = sDataProvider.ReadValue<int>("PublicationSourceKey");
                        document.PublicationDate = sDataProvider.ReadValue<DateTime>("PublicationDate");
                        document.Abstract = sDataProvider.ReadValue<string>("Abstract");
                        document.FileName = sDataProvider.ReadValue<string>("FileName");
                        document.Data = sDataProvider.ReadValue<byte[]>("Data");

                        documentList.Add(document);
                    }

                    return documentList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}