﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class GroupManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the group.
        /// </summary>
        /// <param name="group">The group.</param>
        public static void AddGroup(GroupRecord group)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddGroup";
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, group.UserKey);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, GroupRecord.NameSize, group.Name);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    group.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the group.
        /// </summary>
        /// <param name="group">The group.</param>
        /// <returns></returns>
        public static bool DeleteGroup(GroupRecord group)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteGroup";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, group.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all groups in the database.
        /// </summary>
        /// <returns>The retrieved group collection.</returns>
        public static GroupRecordList LoadGroup()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadGroup";

                return RetrieveMultipleGroups();
            }
        }

        /// <summary>
        /// Retrieve all groups in the database.
        /// </summary>
        /// <param name="userKey"></param>
        /// <returns>The retrieved group collection.</returns>
        public static GroupRecordList LoadGroupByUserKey(int userKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadGroupByUserKey";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);

                return RetrieveMultipleGroups();
            }
        }

        /// <summary>
        /// Retrieves the group by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="group">The group.</param>
        /// <returns></returns>
        public static bool RetrieveGroupByKey(int key, out GroupRecord group)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveGroupByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, key);

                return RetrieveGroup(out group);
            }
        }

        /// <summary>
        /// Retrieves the name of the group by.
        /// </summary>
        /// <param name="userKey">The user key.</param>
        /// <param name="name">The name.</param>
        /// <param name="group">The group.</param>
        /// <returns></returns>
        public static bool RetrieveGroupByName(int userKey, string name, out GroupRecord group)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveGroupByName";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);
                sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, GroupRecord.NameSize, name);

                return RetrieveGroup(out group);
            }
        }

        /// <summary>
        /// Updates the group.
        /// </summary>
        /// <param name="group">The group.</param>
        /// <returns></returns>
        public static bool UpdateGroup(GroupRecord group)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateGroup";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, group.Key);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, group.UserKey);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, GroupRecord.NameSize, group.Name);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the group.
        /// </summary>
        /// <param name="group">The group.</param>
        /// <returns></returns>
        private static bool RetrieveGroup(out GroupRecord group)
        {
            try
            {
                group = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        group = new GroupRecord();
                        group.Key = sDataProvider.ReadValue<int>("Key");
                        group.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        group.Name = sDataProvider.ReadValue<string>("Name");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the groups.
        /// </summary>
        /// <returns></returns>
        private static GroupRecordList RetrieveMultipleGroups()
        {
            GroupRecordList groupList = new GroupRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        GroupRecord group = new GroupRecord();
                        group.Key = sDataProvider.ReadValue<int>("Key");
                        group.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        group.Name = sDataProvider.ReadValue<string>("Name");

                        groupList.Add(group);
                    }

                    return groupList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}