﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class KeywordManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the keyword.
        /// </summary>
        /// <param name="keyword">The keyword.</param>
        public static void AddKeyword(KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddKeyword";
                    sDataProvider.AddParameter("@Word", SqlDbType.VarChar, ParameterDirection.Input, KeywordRecord.WordSize, keyword.Word);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    keyword.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Adds the keyword (or skips it if the word already exists).
        /// </summary>
        /// <param name="keyword">The keyword.</param>
        public static void CreateKeyword(KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "CreateKeyword";
                    sDataProvider.AddParameter("@Word", SqlDbType.VarChar, ParameterDirection.Input, KeywordRecord.WordSize, keyword.Word);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    keyword.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Deletes the keyword.
        /// </summary>
        /// <param name="keyword">The keyword.</param>
        /// <returns></returns>
        public static bool DeleteKeyword(KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteKeyword";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, keyword.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all keywords in the database.
        /// </summary>
        /// <returns>The retrieved keyword collection.</returns>
        public static KeywordRecordList LoadKeyword()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadKeyword";

                return RetrieveMultipleKeywords();
            }
        }

        /// <summary>
        /// Retrieves the keyword by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="keyword">The keyword.</param>
        /// <returns></returns>
        public static bool RetrieveKeywordByKey(int key, out KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveKeywordByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, key);

                return RetrieveKeyword(out keyword);
            }
        }

        /// <summary>
        /// Retrieves the keyword by word.
        /// </summary>
        /// <param name="word">The word.</param>
        /// <param name="keyword">The keyword.</param>
        /// <returns></returns>
        public static bool RetrieveKeywordByWord(string word, out KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveKeywordByWord";
                sDataProvider.AddParameter("@Word", SqlDbType.VarChar, ParameterDirection.Input, KeywordRecord.WordSize, word);

                return RetrieveKeyword(out keyword);
            }
        }

        /// <summary>
        /// Updates the keyword.
        /// </summary>
        /// <param name="keyword">The keyword.</param>
        /// <returns></returns>
        public static bool UpdateKeyword(KeywordRecord keyword)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateKeyword";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, keyword.Key);
                    sDataProvider.AddParameter("@Word", SqlDbType.VarChar, ParameterDirection.Input, KeywordRecord.WordSize, keyword.Word);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the keyword.
        /// </summary>
        /// <param name="keyword">The keyword.</param>
        /// <returns></returns>
        private static bool RetrieveKeyword(out KeywordRecord keyword)
        {
            try
            {
                keyword = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        keyword = new KeywordRecord();
                        keyword.Key = sDataProvider.ReadValue<int>("Key");
                        keyword.Word = sDataProvider.ReadValue<string>("Word");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the keywords.
        /// </summary>
        /// <returns></returns>
        private static KeywordRecordList RetrieveMultipleKeywords()
        {
            KeywordRecordList keywordList = new KeywordRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        KeywordRecord keyword = new KeywordRecord();
                        keyword.Key = sDataProvider.ReadValue<int>("Key");
                        keyword.Word = sDataProvider.ReadValue<string>("Word");

                        keywordList.Add(keyword);
                    }

                    return keywordList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}