﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class PublicationSourceManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the publicationSource.
        /// </summary>
        /// <param name="publicationSource">The publicationSource.</param>
        public static void AddPublicationSource(PublicationSourceRecord publicationSource)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddPublicationSource";
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceRecord.NameSize, publicationSource.Name);
                    sDataProvider.AddParameter("@TypeKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceTypeRecord.KeySize, publicationSource.TypeKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    publicationSource.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the publicationSource.
        /// </summary>
        /// <param name="publicationSource">The publicationSource.</param>
        /// <returns></returns>
        public static bool DeletePublicationSource(PublicationSourceRecord publicationSource)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeletePublicationSource";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, publicationSource.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all publicationSources in the database.
        /// </summary>
        /// <returns>The retrieved publicationSource collection.</returns>
        public static PublicationSourceRecordList LoadPublicationSource()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadPublicationSource";

                return RetrieveMultiplePublicationSources();
            }
        }

        /// <summary>
        /// Loads the publication source by type key.
        /// </summary>
        /// <param name="typeKey">The type key.</param>
        /// <returns></returns>
        public static PublicationSourceRecordList LoadPublicationSourceByTypeKey(int typeKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadPublicationSourceByTypeKey";
                sDataProvider.AddParameter("@TypeKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceTypeRecord.KeySize, typeKey);

                return RetrieveMultiplePublicationSources();
            }
        }

        /// <summary>
        /// Retrieves the publicationSource by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="publicationSource">The publicationSource.</param>
        /// <returns></returns>
        public static bool RetrievePublicationSourceByKey(int key, out PublicationSourceRecord publicationSource)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrievePublicationSourceByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, PublicationSourceRecord.KeySize, key);

                return RetrievePublicationSource(out publicationSource);
            }
        }

        /// <summary>
        /// Retrieves the publicationSource by name.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="publicationSource">The publicationSource.</param>
        /// <returns></returns>
        public static bool RetrievePublicationSourceByName(string name, out PublicationSourceRecord publicationSource)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrievePublicationSourceByName";
                sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceRecord.NameSize, name);

                return RetrievePublicationSource(out publicationSource);
            }
        }

        /// <summary>
        /// Updates the publicationSource.
        /// </summary>
        /// <param name="publicationSource">The publicationSource.</param>
        /// <returns></returns>
        public static bool UpdatePublicationSource(PublicationSourceRecord publicationSource)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdatePublicationSource";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, PublicationSourceRecord.KeySize, publicationSource.Key);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceRecord.NameSize, publicationSource.Name);
                    sDataProvider.AddParameter("@TypeKey", SqlDbType.Int, ParameterDirection.Input, PublicationSourceTypeRecord.KeySize, publicationSource.TypeKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the publicationSource.
        /// </summary>
        /// <param name="publicationSource">The publicationSource.</param>
        /// <returns></returns>
        private static bool RetrievePublicationSource(out PublicationSourceRecord publicationSource)
        {
            try
            {
                publicationSource = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        publicationSource = new PublicationSourceRecord();
                        publicationSource.Key = sDataProvider.ReadValue<int>("Key");
                        publicationSource.Name = sDataProvider.ReadValue<string>("Name");
                        publicationSource.TypeKey = sDataProvider.ReadValue<int>("TypeKey");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the publicationSources.
        /// </summary>
        /// <returns></returns>
        private static PublicationSourceRecordList RetrieveMultiplePublicationSources()
        {
            PublicationSourceRecordList publicationSourceList = new PublicationSourceRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        PublicationSourceRecord publicationSource = new PublicationSourceRecord();
                        publicationSource.Key = sDataProvider.ReadValue<int>("Key");
                        publicationSource.Name = sDataProvider.ReadValue<string>("Name");
                        publicationSource.TypeKey = sDataProvider.ReadValue<int>("TypeKey");

                        publicationSourceList.Add(publicationSource);
                    }

                    return publicationSourceList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}