﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class RepositoryManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the repository.
        /// </summary>
        /// <param name="repository">The repository.</param>
        public static void AddRepository(RepositoryRecord repository)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddRepository";
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, repository.UserKey);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, RepositoryRecord.NameSize, repository.Name);
                    sDataProvider.AddParameter("@Description", SqlDbType.VarChar, ParameterDirection.Input, RepositoryRecord.DescriptionSize, repository.Description);
                    SqlParameter creationDateParameter = sDataProvider.AddOutputParameter("@CreationDate", SqlDbType.DateTime, RepositoryRecord.CreationDateSize);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    repository.Key = (int)returnValue.Value;
                    repository.CreationDate = (DateTime)creationDateParameter.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the repository.
        /// </summary>
        /// <param name="repository">The repository.</param>
        /// <returns></returns>
        public static bool DeleteRepository(RepositoryRecord repository)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteRepository";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, repository.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <returns>The retrieved repository collection.</returns>
        public static RepositoryRecordList LoadRepository()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepository";                

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="userKey">The user key.</param>
        /// <returns>The retrieved repository collection.</returns>
        public static RepositoryRecordList LoadRepositoryByUserKey(int userKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepositoryByUserKey";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);

                return RetrieveMultipleRepositories();
            }
        }

        /// <summary>
        /// Retrieves the repository by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="repository">The repository.</param>
        /// <returns></returns>
        public static bool RetrieveRepositoryByKey(int key, out RepositoryRecord repository)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveRepositoryByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, key);

                return RetrieveRepository(out repository);
            }
        }

        /// <summary>
        /// Retrieves the name of the repository by.
        /// </summary>
        /// <param name="userKey">The user key.</param>
        /// <param name="name">The name.</param>
        /// <param name="repository">The repository.</param>
        /// <returns></returns>
        public static bool RetrieveRepositoryByName(int userKey, string name, out RepositoryRecord repository)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveRepositoryByName";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);
                sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, RepositoryRecord.NameSize, name);

                return RetrieveRepository(out repository);
            }
        }

        /// <summary>
        /// Updates the repository.
        /// </summary>
        /// <param name="repository">The repository.</param>
        /// <returns></returns>
        public static bool UpdateRepository(RepositoryRecord repository)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateRepository";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repository.Key);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.KeySize, repository.UserKey);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, RepositoryRecord.NameSize, repository.Name);
                    sDataProvider.AddParameter("@Description", SqlDbType.VarChar, ParameterDirection.Input, RepositoryRecord.DescriptionSize, repository.Description);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the repository.
        /// </summary>
        /// <param name="repository">The repository.</param>
        /// <returns></returns>
        private static bool RetrieveRepository(out RepositoryRecord repository)
        {
            try
            {
                repository = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        repository = new RepositoryRecord();
                        repository.Key = sDataProvider.ReadValue<int>("Key");
                        repository.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        repository.CreationDate = sDataProvider.ReadValue<DateTime>("CreationDate");
                        repository.Name = sDataProvider.ReadValue<string>("Name");
                        repository.Description = sDataProvider.ReadValue<string>("Description");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the repositories.
        /// </summary>
        /// <returns></returns>
        private static RepositoryRecordList RetrieveMultipleRepositories()
        {
            RepositoryRecordList repositoryList = new RepositoryRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        RepositoryRecord repository = new RepositoryRecord();
                        repository.Key = sDataProvider.ReadValue<int>("Key");
                        repository.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        repository.CreationDate = sDataProvider.ReadValue<DateTime>("CreationDate");
                        repository.Name = sDataProvider.ReadValue<string>("Name");
                        repository.Description = sDataProvider.ReadValue<string>("Description");

                        repositoryList.Add(repository);
                    }

                    return repositoryList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}