﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class ReviewManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the review.
        /// </summary>
        /// <param name="review">The review.</param>
        public static void AddReview(ReviewRecord review)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddReview";
                    sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, review.DocumentKey);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, review.UserKey);
                    sDataProvider.AddParameter("@Rating", SqlDbType.TinyInt, ParameterDirection.Input, ReviewRecord.RatingSize, review.Rating);
                    sDataProvider.AddParameter("@Comments", SqlDbType.Text, ParameterDirection.Input, ReviewRecord.CommentsSize, review.Comments);
                    sDataProvider.AddParameter("@Data", SqlDbType.Image, ParameterDirection.Input, ReviewRecord.DataSize, review.Data);
                    SqlParameter creationDateParameter = sDataProvider.AddOutputParameter("@CreationDate", SqlDbType.DateTime, ReviewRecord.CreationDateSize);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    review.Key = (int)returnValue.Value;
                    review.CreationDate = (DateTime)creationDateParameter.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the review.
        /// </summary>
        /// <param name="review">The review.</param>
        /// <returns></returns>
        public static bool DeleteReview(ReviewRecord review)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteReview";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, review.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all repositories in the database.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns>The retrieved review collection.</returns>
        public static ReviewRecordList LoadReview(int documentKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadReview";
                sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, documentKey);

                return RetrieveMultipleReviews();
            }
        }

        /// <summary>
        /// Retrieves the review by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="review">The review.</param>
        /// <returns></returns>
        public static bool RetrieveReviewByKey(int key, out ReviewRecord review)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveReviewByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, ReviewRecord.KeySize, key);

                return RetrieveReview(out review);
            }
        }

        /// <summary>
        /// Updates the review.
        /// </summary>
        /// <param name="review">The review.</param>
        /// <returns></returns>
        public static bool UpdateReview(ReviewRecord review)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateReview";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, ReviewRecord.KeySize, review.Key);
                    sDataProvider.AddParameter("@Rating", SqlDbType.TinyInt, ParameterDirection.Input, ReviewRecord.RatingSize, review.Rating);
                    sDataProvider.AddParameter("@Comments", SqlDbType.Text, ParameterDirection.Input, ReviewRecord.CommentsSize, review.Comments);
                    sDataProvider.AddParameter("@Data", SqlDbType.Image, ParameterDirection.Input, ReviewRecord.DataSize, review.Data);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the review.
        /// </summary>
        /// <param name="review">The review.</param>
        /// <returns></returns>
        private static bool RetrieveReview(out ReviewRecord review)
        {
            try
            {
                review = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        review = new ReviewRecord();
                        review.Key = sDataProvider.ReadValue<int>("Key");
                        review.DocumentKey = sDataProvider.ReadValue<int>("DocumentKey");                        
                        review.CreationDate = sDataProvider.ReadValue<DateTime>("CreationDate");
                        review.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        review.Rating = sDataProvider.ReadValue<int>("Rating");
                        review.Comments = sDataProvider.ReadValue<string>("Comments");
                        review.Data = sDataProvider.ReadValue<byte[]>("Data");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the repositories.
        /// </summary>
        /// <returns></returns>
        private static ReviewRecordList RetrieveMultipleReviews()
        {
            ReviewRecordList reviewList = new ReviewRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        ReviewRecord review = new ReviewRecord();
                        review.Key = sDataProvider.ReadValue<int>("Key");
                        review.DocumentKey = sDataProvider.ReadValue<int>("DocumentKey");
                        review.CreationDate = sDataProvider.ReadValue<DateTime>("CreationDate");
                        review.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        review.Rating = sDataProvider.ReadValue<int>("Rating");
                        review.Comments = sDataProvider.ReadValue<string>("Comments");
                        review.Data = sDataProvider.ReadValue<byte[]>("Data");

                        reviewList.Add(review);
                    }

                    return reviewList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}