﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class UserManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the user.
        /// </summary>
        /// <param name="user">The user.</param>
        public static void AddUser(UserRecord user)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddUser";
                    sDataProvider.AddParameter("@Username", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.UsernameSize, user.Username);
                    sDataProvider.AddParameter("@Password", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.PasswordSize, user.Password);
                    sDataProvider.AddParameter("@IsSiteAdministrator", SqlDbType.TinyInt, ParameterDirection.Input, UserRecord.IsSiteAdministratorSize, Convert.ToByte(user.IsSiteAdministrator));
                    sDataProvider.AddParameter("@FullName", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.FullNameSize, user.FullName);
                    sDataProvider.AddParameter("@EmailAddress", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.EmailAddressSize, user.EmailAddress);
                    sDataProvider.AddParameter("@Url", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.UrlSize, user.Url);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    user.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the user.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <returns></returns>
        public static bool DeleteUser(UserRecord user)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteUser";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, user.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all users in the database.
        /// </summary>
        /// <returns>The retrieved user collection.</returns>
        public static UserRecordList LoadUser()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadUser";

                return RetrieveMultipleUsers();
            }
        }

        /// <summary>
        /// Retrieves the user by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="user">The user.</param>
        /// <returns></returns>
        public static bool RetrieveUserByKey(int key, out UserRecord user)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveUserByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, key);

                return RetrieveUser(out user);
            }
        }

        /// <summary>
        /// Retrieves the user by username.
        /// </summary>
        /// <param name="username">The username.</param>
        /// <param name="user">The user.</param>
        /// <returns></returns>
        public static bool RetrieveUserByUsername(string username, out UserRecord user)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveUserByUsername";
                sDataProvider.AddParameter("@Username", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.UsernameSize, username);

                return RetrieveUser(out user);
            }
        }

        /// <summary>
        /// Updates the user.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <returns></returns>
        public static bool UpdateUser(UserRecord user)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateUser";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, user.Key);
                    sDataProvider.AddParameter("@Username", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.UsernameSize, user.Username);
                    sDataProvider.AddParameter("@Password", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.PasswordSize, user.Password);
                    sDataProvider.AddParameter("@IsSiteAdministrator", SqlDbType.TinyInt, ParameterDirection.Input, UserRecord.IsSiteAdministratorSize, Convert.ToByte(user.IsSiteAdministrator));
                    sDataProvider.AddParameter("@FullName", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.FullNameSize, user.FullName);
                    sDataProvider.AddParameter("@EmailAddress", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.EmailAddressSize, user.EmailAddress);
                    sDataProvider.AddParameter("@Url", SqlDbType.VarChar, ParameterDirection.Input, UserRecord.UrlSize, user.Url);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the user.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <returns></returns>
        private static bool RetrieveUser(out UserRecord user)
        {
            try
            {
                user = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        user = new UserRecord();
                        user.Key = sDataProvider.ReadValue<int>("Key");
                        user.Username = sDataProvider.ReadValue<string>("Username");
                        user.Password = sDataProvider.ReadValue<string>("Password");
                        user.IsSiteAdministrator = sDataProvider.ReadValue<bool>("IsSiteAdministrator");
                        user.FullName = sDataProvider.ReadValue<string>("FullName");
                        user.EmailAddress = sDataProvider.ReadValue<string>("EmailAddress");
                        user.Url = sDataProvider.ReadValue<string>("Url");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the users.
        /// </summary>
        /// <returns></returns>
        private static UserRecordList RetrieveMultipleUsers()
        {
            UserRecordList userList = new UserRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        UserRecord user = new UserRecord();
                        user.Key = sDataProvider.ReadValue<int>("Key");
                        user.Username = sDataProvider.ReadValue<string>("Username");
                        user.Password = sDataProvider.ReadValue<string>("Password");
                        user.IsSiteAdministrator = sDataProvider.ReadValue<bool>("IsSiteAdministrator");
                        user.FullName = sDataProvider.ReadValue<string>("FullName");
                        user.EmailAddress = sDataProvider.ReadValue<string>("EmailAddress");
                        user.Url = sDataProvider.ReadValue<string>("Url");

                        userList.Add(user);
                    }

                    return userList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}